require 'gradesfirst/command'

module GradesFirst
  class CodeTalkCommand < GradesFirst::Command
    # Description of the "gf codetalk" command that will be used for the
    # commandline help.
    def self.description
      'Show the next dates for code talks.'
    end

    # Description of the "gf codetalk" command that will be used in the
    # commandline help to provide a complete description.
    def self.long_description
      <<-LONG_DESCRIPTION
      Code talk dates are calculated based on a zero day date and the list
      of people who can give code talks.  The dates are based on a simple
      rotation.

      In order for this command to work correctly, the environment variables
      GF_DAY_ZERO and GF_CODE_TALKERS need to be properly defined.  These can
      be actual environment variables, a .env file, or .env.master file.  The
      .env file can be in the current directory or the home directory.  The
      .env.master file must be in the current directory.  The precedence order
      is environment variables, .env in the current directory, .env in the
      home directory, and .env.master in the current directory.  It is
      recommended that .env.master be committed to source control and the
      other options used to override .env.master.

      GF_DAY_ZERO is a date in a valid Ruby date string format.

      GF_CODE_TALKERS is a pipe (|) delimited list of names.

      Examples:

      GF_DAY_ZERO="2014-3-10 00:00:00"

      GF_CODE_TALKERS="Anthony Crumley|Tom Miller|Andrew Sellers"
      LONG_DESCRIPTION
    end

    # Performs the gf codetalk command with show the code talk assignments
    # for the next round of talks.
    def execute
      @assignments = {}
      code_talkers.count.times { |index| @assignments[index] = next_talk_date }
    end

    # Generates the command line output response.  The output of the codetalk
    # command will be the name and next date for each code talker.
    def response
      @assignments.
        map { |talker_index, date| assignment(talker_index, date) }.
        join("\n").
        concat("\n")
    end

    private

    def assignment(talker_index, date)
      "#{date.strftime("%m/%d/%Y %a")} - #{next_code_talkers[talker_index]}"
    end

    def code_talkers
      config.code_talkers.sort
    end

    def next_code_talkers
      @next_code_talkers ||= code_talkers.rotate(talk_count % code_talkers.count)
    end

    def next_talk_date
      if @next_talk_date
        @next_talk_date = @next_talk_date.next_day
      else
        @next_talk_date = today
      end

      until @next_talk_date.thursday?
        @next_talk_date = @next_talk_date.next_day
      end

      @next_talk_date
    end

    def talk_count
      talk_range.reduce(0) do |count, date|
        count += 1 if date.thursday?
        count
      end
    end

    def talk_range
      day_zero.to_date..today.prev_day
    end

    def today
      Date.today
    end
  end
end
